::This file was made by inoage GmbH, http://www.inoage.de/

::Usage: build-Qwt-6.2-VC16-x64.bat [/code=code-dir] [/build=build-dir] [/deploy=deploy-dir] [/logs=logs-dir] [/qt=qt-dir] [/t:target]
::Builds Qwt 6.2 (x64) for VC16

::Hints:
::The source code directory must not contain spaces in the path
::Qwt 6.2 supports out-of-source builds

::References:
::https://qwt.sourceforge.io/qwtinstall.html

@echo off
setlocal enabledelayedexpansion

::Setup the environment
set repoDir=..\..\..\..
set PATH=%repoDir%\sdk\utility;%PATH%
call "C:\Program Files (x86)\Microsoft Visual Studio\2019\Enterprise\VC\Auxiliary\Build\vcvarsall.bat" amd64 2> nul || ^
call "C:\Program Files (x86)\Microsoft Visual Studio\2019\Professional\VC\Auxiliary\Build\vcvarsall.bat" amd64 2> nul || (
echo The setup script for the Visual Compiler environment ^(VC16^) could not be found^^! & pause & exit /b 1)
echo.

::Parse the parameters
set defaultBaseDir=C:\SDK\Qwt\Qwt-6.2.0
set codeDir=%defaultBaseDir%\code
set buildDir=%defaultBaseDir%\build\VC16\x64
set deployDir=%defaultBaseDir%\deploy\VC16\x64
set logsDir=
set qtDir=C:\SDK\Qt\Qt-5.9.7\deploy\VC16\x64
set defaultTargets=:clean-up-build :clean-up-deploy :clean-up-logs :config :build :deploy
set targets=%defaultTargets%

:beginParseParameters
set p=%1
if _%p%_==__ goto :endParseParameters
if /i _%p%_==_/code_      (shift & set codeDir=%~f2) else ^
if /i _%p%_==_/build_     (shift & set buildDir=%~f2) else ^
if /i _%p%_==_/deploy_    (shift & set deployDir=%~f2) else ^
if /i _%p%_==_/logs_      (shift & set logsDir=%~f2) else ^
if /i _%p%_==_/qt_        (shift & set qtDir=%~f2) else ^
if /i _%p%_==_/t:clean-up-build_  (set targets=%p:~2%) else ^
if /i _%p%_==_/t:clean-up-deploy_ (set targets=%p:~2%) else ^
if /i _%p%_==_/t:clean-up-logs_   (set targets=%p:~2%) else ^
if /i _%p%_==_/t:build_           (set targets=%p:~2%) else ^
if /i _%p%_==_/t:deploy_          (set targets=%p:~2%) else ^
if /i _%p%_==_/t:clean-up_        (set targets=:clean-up-build :clean-up-deploy :clean-up-logs) else ^
if /i _%p%_==_/t:reconfig_        (set targets=:clean-up-build :clean-up-deploy :clean-up-logs :config) else ^
if /i _%p%_==_/t:rebuild_         (set targets=:clean-up-build :clean-up-deploy :clean-up-logs :config :build) else ^
if /i _%p%_==_/t:build-deploy_    (set targets=:build :deploy) else ^
if /i _%p%_==_/t:all_             (set targets=%defaultTargets%) else ^
if /i _%p%_==_/t:all-tidy_        (set targets=%defaultTargets% :clean-up-build) else ^
echo Skipping the invalid parameter '%p%'...
shift
goto :beginParseParameters
:endParseParameters

if "%logsDir%"=="" set logsDir=%deployDir%\logs

echo Description: Builds Qwt 6.2 ^(x64^) for VC16
echo Source code: '%codeDir%'
echo Build:       '%buildDir%'
echo Deploy:      '%deployDir%'
echo Logs:        '%logsDir%'
echo Targets:     '%targets%'
echo.
echo Qt:          '%qtDir%'
"%qtDir%\bin\qmake.exe" -v || (echo The executable 'qmake.exe' could not be found in the Qt binary directory^^! & pause & exit /b 1)
echo.
echo These settings have been deduced from command line parameters or set to the default values.
set /p c=Do you want to continue [yn]? & if _!c!_==__ set c=y
if /i not _%c%_==_y_ (echo Canceled. & exit /b 0)
echo.

::Call the targets
for %%I in (%targets%) do (call %%I || (pause & exit /b 1))
pause & exit /b 0

:clean-up-build
if not exist "%buildDir%" goto :eof
echo Cleaning up the build directory...
call rmPath.bat "%buildDir%"
echo.
goto :eof

:clean-up-deploy
if not exist "%deployDir%" goto :eof
echo Cleaning up the deploy directory...
call rmPath.bat "%deployDir%"
echo.
goto :eof

:clean-up-logs
if not exist "%logsDir%" goto :eof
echo Cleaning up the logs directory...
call rmPath.bat "%logsDir%"
echo.
goto :eof

:config
::Check for the source code directory
if not exist "%codeDir%" (echo The source code directory could not be found^^! & exit /b 1)
if not exist "%codeDir%\qwt.pro" (echo The project 'qwt.pro' could not be found in the source code directory^^! & exit /b 1)

echo Creating the build, deploy and logs directories...
mkdir "%buildDir%"
mkdir "%deployDir%"
mkdir "%logsDir%"
echo.

echo Configuring the flags...
set logFile=%logsDir%\configure.log
set logFileV=%logsDir%\configure_verbose.log
echo See '%logFile%'
echo and '%logFileV%' for details.
set configureFlags=

::Target parameters
set configureFlags=%configureFlags% -spec win32-msvc

::Show all warnings
set configureFlags=%configureFlags% -Wall

::Prepare some environment variables for qmake. Other flags are configured 
::directly in "codeDir\qwtconfig.pri".
::Deploy out-of-source
set QWT_INSTALL_PREFIX=%deployDir:\=/%

::Configure out-of-source
cd /d "%buildDir%"
"%qtDir%\bin\qmake.exe" %configureFlags% "%codeDir%\qwt.pro" > "%logFileV%" 2> "%logFile%"
if not %errorlevel%==0 (echo Failed^^! & exit /b 1)
echo.
goto :eof

:build
echo Compiling...
set logFile=%logsDir%\compile.log
echo See '%logFile%' for details.

::Build out-of-source
cd /d "%buildDir%"

::Options: /nologo ... Suppress subsequent copyright messages
::         /s ........ Silent mode
::         /x file ... Redirect warnings and errors to file
set innerCommand=nmake /nologo /s /x "%logFile%"

::Options: /high ..... Schedule with high priority
::         /wait ..... Wait for the inner command to finish
start /high /wait %innerCommand%
if not %errorlevel%==0 (echo Failed^^! & exit /b 1)
echo.
goto :eof

:deploy
echo Installing...
set logFile=%logsDir%\install.log
echo See '%logFile%' for details.

::Deploy out-of-source
cd /d "%buildDir%"

::Options: See above
set innerCommand=nmake /nologo /s /x "%logFile%" install

::Options: See above
start /high /wait %innerCommand%
if not %errorlevel%==0 (echo Failed^^! & exit /b 1)
echo.
goto :eof
